import os

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

try:
    import uncloud.version

    version = uncloud.version.VERSION
except:
    import subprocess

    c = subprocess.check_output(["git", "describe"])
    version = c.decode("utf-8").strip()


setup(
    name="uncloud",
    version=version,
    description="uncloud cloud management",
    url="https://code.ungleich.ch/uncloud/uncloud",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python :: 3",
    ],
    author="ungleich",
    author_email="technik@ungleich.ch",
    packages=find_packages(),
    install_requires=[
        "requests",
        "Flask>=1.1.1",
        "flask-restful",
        "bitmath",
        "pyotp",
        "pynetbox",
        "colorama",
        "etcd3 @ https://github.com/kragniz/python-etcd3/tarball/master#egg=etcd3",
        "marshmallow",
        "ldap3"
    ],
    scripts=["scripts/uncloud"],
    data_files=[
        (os.path.expanduser("~/uncloud/"), ["conf/uncloud.conf"])
    ],
    zip_safe=False,
)
